﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.Scheduling.Interfaces;
using HeuristicLab.Operators;

namespace HeuristicLab.Problems.Scheduling.Encodings {
  [Item("Scheduling Decoder", "A scheduling decoder operator.")]
  [StorableClass]
  public abstract class SchedulingDecoder<T> : SingleSuccessorOperator, ISchedulingDecoder where T : Item, ISchedulingEncoding {
    [StorableConstructor]
    protected SchedulingDecoder(bool deserializing) : base(deserializing) { }
    protected SchedulingDecoder(SchedulingDecoder<T> original, Cloner cloner)
      : base(original, cloner) {
    }

    public ILookupParameter<T> SchedulingSolutionParameter {
      get { return (ILookupParameter<T>)Parameters["SchedulingSolution"]; }
    }
    public ILookupParameter<Schedule> DecodedSchedulingSolutionParameter {
      get { return (ILookupParameter<Schedule>)Parameters["DecodedSchedulingSolution"]; }
    }

    public SchedulingDecoder()
      : base() {
      Parameters.Add(new LookupParameter<T>("SchedulingSolution", "The new scheduling solution represented as encoding."));
      Parameters.Add(new LookupParameter<Schedule>("DecodedSchedulingSolution", "The decoded scheduling solution represented as generalized schedule."));
    }

    public abstract Schedule CreateScheduleFromEncoding(T solution);

    public override IOperation Apply() {
      Schedule result = CreateScheduleFromEncoding(SchedulingSolutionParameter.ActualValue);
      DecodedSchedulingSolutionParameter.ActualValue = result;
      return base.Apply();
    } 
  }
}
