﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Data;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.Scheduling {
  [Item("Job Class", "Represents a composition of tasks that require processing in a job shop scheduling problem.")]
  [StorableClass]
  public class JSSPJob : IndexedTaskList {
    [StorableConstructor]
    protected JSSPJob(bool deserializing) : base(deserializing) { }
    protected JSSPJob(JSSPJob original, Cloner cloner)
      : base(original, cloner) {
        this.DueDate = cloner.Clone(original.DueDate);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new JSSPJob(this, cloner);
    }

    public DoubleValue DueDate { get; set; }

    public JSSPJob(IntValue index, DoubleValue dueDate) : base (index) {
      if (dueDate != null)
        DueDate = (DoubleValue)dueDate.Clone();
      else
        DueDate = null;
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("Job#" + Index + " [ ");
      foreach (JSSPTask t in Tasks) {
        sb.Append(t.ToString () + " ");
      }
      if (DueDate != null)
        sb.Append("{" + DueDate.Value + "} ");
      sb.Append("]");
      return sb.ToString();
    }

    internal JSSPTask GetPreviousTask(JSSPTask t) {
      if (t.TaskNr.Value == 0)
        return null;
      else
        return Tasks[t.TaskNr.Value - 1];
    }
  }
}
